// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include <string>

#include <Dal.h>
#include <Tracer.h>
#include <errstr.h>
#include <SasDalUtils.h>

#include "CcfConstituent.h"
#include "CcfConstituentFactory.h"

#include "CalIndex.h"

#include "CalIndexSet.h"
#include "CalIndexColumns.h"  

static const string calindex = "CALINDEX";

CalIndexSet::CalIndexSet(const string& name, const bool& master,
			 const bool& fullpath, const string& category,
			 const STime& analysisdate,
			 const STime& observationdate)
  :
  _can_insert(true),
  _fullpath(fullpath),
  _master(master),
  _category(category)
{
  PROC

  _set = dataSetServer->open(name, Dal::Create);

  //  *_set->addAttribute("OBSVDATE") = observationdate.toString(0);
  //  *_set->addAttribute("ANALDATE") = analysisdate.toString(0);
  if(master){
    // Write in the mandatory set attributes to make this a valid CCF
    // constituent.
    *_set->addAttribute("ORIGIN") = "XMM-Newton Science Operations Centre";
    *_set->addAttribute("CATEGORY") = _category;
    string fname = name;
    string::size_type n = name.find_last_of("/");
    if(n != string::npos)
      fname = name.substr(n+1);
    *_set->addAttribute("FILENAME") = fname;
    *_set->addAttribute("TELESCOP") = "XMM";
    *_set->addAttribute("SCOPE") = "XMM";
    *_set->addAttribute("TYPEID") = "CALINDEX";
    *_set->addAttribute("ISSUE") = 1;
    *_set->addAttribute("VALDATE") = STime("now").toString(0);
    *_set->addAttribute("SUBDATE") = STime("now").toString(0);
    *_set->addAttribute("EXTSEQID") = calindex;
    *_set->addAttribute("EXTSEQU") = "ANY";
  }
  
  Table * table = _set->addTable(calindex, 0, "");
  * table->addAttribute("CATEGORY") = _category;
  // duplicated here as the CALINDEX block is likely to be exported out
  if(!master){
    * table->addAttribute("OBSVDATE") = observationdate.toString(0);
    * table->addAttribute("ANALDATE") = analysisdate.toString(0);
  }
  vector<unsigned long> size(1);

  size[0] = 4;
  table->addColumn(TELESCOPE, Dal::DString, "Telescope", "", size);
  
  size[0] = 6;
  table->addColumn(SCOPE, Dal::DString, "Calibration scope", "", size);
  
  size[0] = 32;
  table->addColumn(TYPE_ID, Dal::DString, "Calibration data type",	"", size);
  
  size[0] = 1;
  table->addColumn(ISSUE, Dal::Int16, "Constituent issue number", "", size);
  
  size[0] = 19;
  table->addColumn(VALDATE, Dal::DString, "Start of validity", "yyyy:dd:mmZhh:mm:ss", size);
  
  size[0] = 19;
  table->addColumn(ENDVALDATE, Dal::DString, "End of validity", "yyyy:dd:mmZhh:mm:ss", size);
  
  size[0] = 256;
  table->addColumn(NAME, Dal::DString, "Constituent file path", "", size);
  
  size[0] = 19;
  table->addColumn(DATE, Dal::DString, "Creation date", "yyyy:dd:mmZhh:mm:ss", size);
  
  size[0] = 1;
  table->addColumn(FSIZE, Dal::Int32, "Constituent size", "byte", size);
  
  size[0] = 19;
  table->addColumn(SUBDATE, Dal::DString, "Submission date", "yyyy:dd:mmZhh:mm:ss", size);
  
  size[0] = 32;
  table->addColumn(EXT_SEQUENCE, Dal::DString, "Extension sequence", "", size);
  
  size[0] = 256;
  table->addColumn(EXT_SEQUENCE_ID, Dal::DString, "Extension sequence identifiers", "", size);
  
  size[0] = 32;
  table->addColumn(MD5_SIGNATURE, Dal::DString, "MD5 digital signature", "", size);
  
  size[0] = 64;
  table->addColumn(CREATOR, Dal::DString, "File creator", "", size);
  
  ENDPROC
}

CalIndexSet::CalIndexSet(const string& name, const bool& readonly)
  : _can_insert(false)
{
  PROC
  _set = dataSetServer->open(name, readonly ? Dal::Read : Dal::Modify);
  if(!_set->hasBlock(calindex))
    errstr << name + " is not a Calibration Index File." << Sas::error("TableNotFound");
  ENDPROC
}

CalIndex CalIndexSet::calibrationIndex(const string& name) const
{
  PROC
  CalIndex cif(name);
  for(unsigned long i = 0; i < size(); i++)
    cif.addConstituent(getConstituent(i));
  cif.sort(CcfConstituent::compare);
  RETURN cif;
}

void CalIndexSet::insert(CcfConstituent * c)
{
  PROC
  _set->table(calindex)->insertRows(_set->table(calindex)->rows());
  unsigned long row = _set->table(calindex)->rows() - 1;

  ColumnData * ptr;

  ptr = _set->table(calindex)->column(FSIZE)->data();
  int32 * s = ptr->int32Data();
  s[row] = c->size();
  delete ptr;
  
  ptr = _set->table(calindex)->column(ISSUE)->data();
  int16 * i = ptr->int16Data();
  i[row] = c->issue();
  delete ptr;
  
  using namespace SasDalUtils;
  ptr = _set->table(calindex)->column(TELESCOPE)->data();
  setStringColumnRow(ptr, row , c->telescope());
  delete ptr;
  
  ptr = _set->table(calindex)->column(SCOPE)->data();
  setStringColumnRow(ptr, row , c->scope());
  delete ptr;
  
  ptr = _set->table(calindex)->column(TYPE_ID)->data();
  setStringColumnRow(ptr, row , c->type());
  delete ptr;
  
  ptr = _set->table(calindex)->column(VALDATE)->data();
  setStringColumnRow(ptr, row , c->valdate().toString());
  delete ptr;

  ptr = _set->table(calindex)->column(ENDVALDATE)->data();
  if(c->hasEndvaldate())
    setStringColumnRow(ptr, row , c->endvaldate().toString());
  else
    setStringColumnRow(ptr, row , std::string(19, ' '));    
  delete ptr;
  
  ptr = _set->table(calindex)->column(SUBDATE)->data();
  setStringColumnRow(ptr, row , c->subdate().toString());
  delete ptr;
  
  ptr = _set->table(calindex)->column(DATE)->data();
  setStringColumnRow(ptr, row , c->date().toString());
  delete ptr;
  
  ptr = _set->table(calindex)->column(EXT_SEQUENCE)->data();
  setStringColumnRow(ptr, row , c->extSequence());
  delete ptr;
  
  ptr = _set->table(calindex)->column(EXT_SEQUENCE_ID)->data();
  setStringColumnRow(ptr, row , c->extSequenceIdentifier());
  delete ptr;
  
  ptr = _set->table(calindex)->column(NAME)->data();
  _fullpath ? setStringColumnRow(ptr, row , c->name()) :
    setStringColumnRow(ptr, row , c->basename());
  delete ptr;
  
  ptr = _set->table(calindex)->column(CREATOR)->data();
  setStringColumnRow(ptr, row , c->creator());
  delete ptr;
  
  ptr = _set->table(calindex)->column(MD5_SIGNATURE)->data();
  c->sign();
  setStringColumnRow(ptr, row, c->signature());
  delete ptr;

  ENDPROC
}

void CalIndexSet::close()
{
  PROC
  dataSetServer->close(_set);
  _can_insert = false;
  ENDPROC
}

CcfConstituent * CalIndexSet::getConstituent(const unsigned int& row) const
{
  PROC
  RETURN(ccfConstituentFactory->ccfConstituent(_set->table(calindex), row));
}

unsigned long CalIndexSet::size() const
{
  PROC
  RETURN(_set->table(calindex)->rows());
}
