// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// testcifbuild -- Giuseppe Vacanti, XMM-SOC, November 1999
//
// Relies on the dates in the dummy constituents in mytestccf.
//
#include <string>
#include <Dal.h>
#include <cifbuild.h>
#include <param.h>
#include <metatask.h>
#include <CalIndexSet.h>
#include <cassert>
#include <errstr.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

void create_ccf_component( string name, string scope, string type, int issue,
			   string valdate, string subdate, string category = "XMMCCF" )
{ 
  DataSet *set = dataSetServer->open(name, Dal::Create);
  *set->addAttribute("TELESCOP") = "XMM";
  *set->addAttribute("INSTRUME") = "XMM";
  *set->addAttribute("SCOPE") = scope;
  *set->addAttribute("TYPEID") = type;
  *set->addAttribute("TYPEDESC") = type;
  *set->addAttribute("ISSUE") = issue;
  *set->addAttribute("VALDATE") = valdate;
  *set->addAttribute("SUBDATE") = subdate;
  *set->addAttribute("EXTSEQU") = "ANY";
  *set->addAttribute("EXTSEQID") = "NONE";
  *set->addAttribute("CATEGORY") = category;
  dataSetServer->close(set);
}

int main() {
  
  const string t1 = "1998-01-01T00:00:00";
  const string t2 = "1998-02-01T00:00:00";
  const string t3 = "1998-03-01T00:00:00";
  const string cif = "ccf.cif";
  const string mif = "ccf.mif";
  const string dir = "mytestccf";

  // first pass
  cerr << "first pass" << endl;
  openParameters("cifbuild");
  setParameter("withccfpath", true);
  setParameter("ccfpath",dir.c_str());
  setParameter("withobservationdate",true);
  setParameter("analysisdate",t1);
  setParameter("calindexset", cif);
  setParameter("observationdate",t1);
  setParameter("category","XMMCCF");
  task(cifbuild);

  CalIndexSet * cis;
  cis = new CalIndexSet(cif);
  assert(cis->size() == 4);

  assert(cis->getConstituent(3)->type() == "CTI");
  assert(cis->getConstituent(3)->issue() == 1);
  assert(cis->getConstituent(0)->type() == "ABSCOEFS");
  assert(cis->getConstituent(0)->issue() == 3);
  cis->close();

  // check that the correct table name and attributes are there
  DataSet *set = dataSetServer->open(cif, Dal::Read);
  Table *tab = set->table("CALINDEX");
  tab->attribute("OBSVDATE");
  tab->attribute("ANALDATE");
  dataSetServer->close(set);

  // second pass
  cerr << "second pass" << endl;
  setParameter("withobservationdate",true);
  setParameter("analysisdate","now");
  setParameter("observationdate","now");
  task(cifbuild);

  cis = new CalIndexSet(cif);
  assert(cis->size() == 4);

  assert(cis->getConstituent(0)->type() == "ABSCOEFS");
  assert(cis->getConstituent(0)->issue() == 4);
  cis->close();

  // third pass
  cerr << "third pass" << endl;
  setParameter("withobservationdate",true);
  setParameter("category","SCISIMCCF");
  task(cifbuild);

  cis = new CalIndexSet(cif);
  assert(cis->size() == 1);
  cis->close();

  // fourth pass
  cerr << "fourth pass" << endl;
  setParameter("withobservationdate",true);
  setParameter("category","XMMCCF");
  setParameter("append",true);
  task(cifbuild);

  cis = new CalIndexSet(cif);
  assert(cis->size() == 4);
  cis->close();

  // fifth pass
  cerr << "fifth pass" << endl;
  setParameter("withobservationdate",true);
  setParameter("append",false);
  setParameter("withccfpath", true);
  setParameter("ccfpath",".");
  setParameter("calindexset", cif);
  task(cifbuild);
  cis = new CalIndexSet(cif);
  assert(cis->size() == 0);
  cis->close();
 
  delete cis;

  closeParameters();

  // MIF: this is like the first pass, but we first go through a MIF
  cerr << "mif pass" << endl;
  openParameters("cifbuild");
  setParameter("withccfpath", true);
  setParameter("ccfpath", dir.c_str());
  setParameter("masterindex", true);
  setParameter("calindexset", mif);
  task(cifbuild);
  closeParameters();

  openParameters("cifbuild");
  setParameter("withmasterindexset", true);
  setParameter("masterindexset", mif);
  setParameter("withobservationdate",true);
  setParameter("analysisdate",t1);
  setParameter("calindexset", cif);
  setParameter("observationdate",t1);
  setParameter("category","XMMCCF");
  task(cifbuild);

  cis = new CalIndexSet(cif);
  assert(cis->size() == 4);

  assert(cis->getConstituent(3)->type() == "CTI");
  assert(cis->getConstituent(3)->issue() == 1);
  assert(cis->getConstituent(0)->type() == "ABSCOEFS");
  assert(cis->getConstituent(0)->issue() == 3);
  cis->close();

// check that the correct table name and attributes are there
//    DataSet *set = dataSetServer->open(cif, Dal::Read);
//    Table *tab = set->table("CALINDEX");
//    tab->attribute("OBSVDATE");
//    tab->attribute("ANALDATE");
//    dataSetServer->close(set);

  unlink(cif.c_str());
  unlink(mif.c_str());

}
  
